# {{ title }}

{% if timestamp_line -%}
{{ timestamp_line }}

{% endif -%}
{% if dual_style -%}
{{ dual_style }}{% endif -%}
{% set title = "Map" %}
{% set body = map_block %}
{% include "markdown_section.md.j2" %}

{% if legend_title -%}
{% set title = legend_title %}
{% set body = legend_block %}
{% include "markdown_section.md.j2" %}

{% else -%}
{{ legend_block }}

{% endif -%}

{{ device_overview }}
