.unifi-legend-hidden,
.unifi-legend-hidden.unifi-legend,
.unifi-legend-hidden.unifi-legend--light,
.unifi-legend-hidden.unifi-legend--dark {
  display: none !important;
}
.unifi-legend-sidebar {
  margin-top: 1rem;
  padding: 0.5rem 0.75rem;
  border: 1px solid rgba(0, 0, 0, 0.08);
  border-radius: 6px;
  font-size: 0.75rem;
}
.unifi-legend-title {
  font-weight: 600;
  margin-bottom: 0.5rem;
}
.unifi-legend-sidebar table {
  width: 100%;
  border-collapse: collapse;
}
.unifi-legend-sidebar td,
.unifi-legend-sidebar th {
  border: 0;
  padding: 0.15rem 0;
}
.unifi-legend-sidebar svg {
  display: block;
}
