"""Generate mock UniFi data using Faker."""

from __future__ import annotations

from ..model.mock import MockOptions, generate_mock_payload, mock_payload_json

__all__ = ["MockOptions", "generate_mock_payload", "mock_payload_json"]
