## {{ device_name }}

{{ details }}
{% if ports_section %}

{{ ports_section }}
{% endif %}
{% if lldp_section %}

{{ lldp_section }}
{% endif %}
{% if clients_section %}

{{ clients_section }}
{% endif %}
