<style>
.unifi-mermaid--light,.unifi-legend--light{display:none;}
.unifi-mermaid--dark,.unifi-legend--dark{display:none;}
[data-md-color-scheme="default"] .unifi-mermaid--light{display:block;}
[data-md-color-scheme="default"] .unifi-legend--light{display:block;}
[data-md-color-scheme="slate"] .unifi-mermaid--dark{display:block;}
[data-md-color-scheme="slate"] .unifi-legend--dark{display:block;}
</style>
